@@ -1,7 +1,7 @@
 """The implementation of a Hera cron workflow for Argo-based cron workflows"""
 from datetime import datetime
 from datetime import timezone as tz
-from typing import Dict, Optional, Tuple
+from typing import Dict, List, Optional, Tuple
 from uuid import uuid4
 
 import pytz
@@ -12,10 +12,13 @@ from argo_workflows.models import (
     IoArgoprojWorkflowV1alpha1DAGTemplate,
     IoArgoprojWorkflowV1alpha1Template,
     IoArgoprojWorkflowV1alpha1WorkflowSpec,
+    IoArgoprojWorkflowV1alpha1WorkflowTemplateRef,
+    LocalObjectReference,
     ObjectMeta,
 )
 
 from hera.cron_workflow_service import CronWorkflowService
+from hera.security_context import WorkflowSecurityContext
 from hera.task import Task
 from hera.volumes import Volume
 
@@ -44,6 +47,16 @@ class CronWorkflow:
         A Dict of labels to attach to the CronWorkflow object metadata.
     namespace: Optional[str] = 'default'
         The namespace to use by default when calling create/suspend/resume.  Defaults to 'default'.
+    security_context:  Optional[WorkflowSecurityContext] = None
+        Define security settings for all containers in the workflow.
+    image_pull_secrets: Optional[List[str]] = None
+        A list of image pull secrets. This is used to authenticate with the private image registry of the images
+        used by tasks.
+    workflow_template_ref: Optional[str] = None
+        The name of the workflowTemplate reference. WorkflowTemplateRef is a reference to a WorkflowTemplate resource.
+        If you create a WorkflowTemplate resource either clusterWorkflowTemplate or not (clusterScope attribute bool)
+        you can reference it again and again when you create a new Workflow without specifying the same tasks and
+        dependencies. Official doc: https://argoproj.github.io/argo-workflows/fields/#workflowtemplateref
     """
 
     def __init__(
@@ -56,6 +69,9 @@ class CronWorkflow:
         service_account_name: Optional[str] = None,
         labels: Optional[Dict[str, str]] = None,
         namespace: Optional[str] = "default",
+        security_context: Optional[WorkflowSecurityContext] = None,
+        image_pull_secrets: Optional[List[str]] = None,
+        workflow_template_ref: Optional[str] = None,
     ):
         if timezone and timezone not in pytz.all_timezones:
             raise ValueError(f'{timezone} is not a valid timezone')
@@ -68,6 +84,9 @@ class CronWorkflow:
         self.service_account_name = service_account_name
         self.labels = labels
         self.namespace = namespace
+        self.security_context = security_context
+        self.image_pull_secrets = image_pull_secrets
+        self.workflow_template_ref = workflow_template_ref
 
         self.dag_template = IoArgoprojWorkflowV1alpha1DAGTemplate(tasks=[])
         self.template = IoArgoprojWorkflowV1alpha1Template(
@@ -76,13 +95,32 @@ class CronWorkflow:
             dag=self.dag_template,
             parallelism=self.parallelism,
         )
-        self.spec = IoArgoprojWorkflowV1alpha1WorkflowSpec(
-            templates=[self.template], entrypoint=self.name, volumes=[], volume_claim_templates=[]
-        )
+
+        if self.workflow_template_ref:
+            self.workflow_template = IoArgoprojWorkflowV1alpha1WorkflowTemplateRef(name=self.workflow_template_ref)
+            self.spec = IoArgoprojWorkflowV1alpha1WorkflowSpec(
+                workflow_template_ref=self.workflow_template,
+                entrypoint=self.workflow_template_ref,
+                volumes=[],
+                volume_claim_templates=[],
+            )
+        else:
+            self.spec = IoArgoprojWorkflowV1alpha1WorkflowSpec(
+                templates=[self.template], entrypoint=self.name, volumes=[], volume_claim_templates=[]
+            )
+
         if self.service_account_name:
             setattr(self.template, 'service_account_name', self.service_account_name)
             setattr(self.spec, 'service_account_name', self.service_account_name)
 
+        if self.security_context:
+            security_context = self.security_context.get_security_context()
+            setattr(self.spec, 'security_context', security_context)
+
+        if self.image_pull_secrets:
+            secret_refs = [LocalObjectReference(name=name) for name in self.image_pull_secrets]
+            setattr(self.spec, 'image_pull_secrets', secret_refs)
+
         self.cron_spec = IoArgoprojWorkflowV1alpha1CronWorkflowSpec(schedule=self.schedule, workflow_spec=self.spec)
         if self.timezone:
             setattr(self.cron_spec, 'timezone', self.timezone)
