@@ -1,5 +1,6 @@
 """The implementation of a Hera workflow for Argo-based workflows"""
-from typing import Dict, Optional
+import warnings
+from typing import Dict, List, Optional
 from uuid import uuid4
 
 from argo_workflows.models import (
@@ -7,6 +8,8 @@ from argo_workflows.models import (
     IoArgoprojWorkflowV1alpha1Template,
     IoArgoprojWorkflowV1alpha1Workflow,
     IoArgoprojWorkflowV1alpha1WorkflowSpec,
+    IoArgoprojWorkflowV1alpha1WorkflowTemplateRef,
+    LocalObjectReference,
     ObjectMeta,
 )
 
@@ -15,6 +18,10 @@ from hera.task import Task
 from hera.volumes import Volume
 from hera.workflow_service import WorkflowService
 
+# by default, `DeprecationWarning`s are silenced, this removes the warning from the filter so it
+# can be issued to users
+warnings.simplefilter('always', DeprecationWarning)
+
 
 class Workflow:
     """A workflow representation.
@@ -34,12 +41,20 @@ class Workflow:
         The number of parallel tasks to run in case a task group is executed for multiple tasks.
     service_account_name: Optional[str] = None
         The name of the service account to use in all workflow tasks.
-    security_context:  Optional[WorkflowSecurityContext] = None
-        Define security settings for all containers in the workflow.
     labels: Optional[Dict[str, str]] = None
         A Dict of labels to attach to the Workflow object metadata
     namespace: Optional[str] = 'default'
         The namespace to use for creating the Workflow.  Defaults to "default"
+    security_context:  Optional[WorkflowSecurityContext] = None
+        Define security settings for all containers in the workflow.
+    image_pull_secrets: Optional[List[str]] = None
+        A list of image pull secrets. This is used to authenticate with the private image registry of the images
+        used by tasks.
+    workflow_template_ref: Optional[str] = None
+        The name of the workflowTemplate reference. WorkflowTemplateRef is a reference to a WorkflowTemplate resource.
+        If you create a WorkflowTemplate resource either clusterWorkflowTemplate or not (clusterScope attribute bool)
+        you can reference it again and again when you create a new Workflow without specifying the same tasks and
+        dependencies. Official doc: https://argoproj.github.io/argo-workflows/fields/#workflowtemplateref
     """
 
     def __init__(
@@ -51,6 +66,8 @@ class Workflow:
         labels: Optional[Dict[str, str]] = None,
         namespace: Optional[str] = None,
         security_context: Optional[WorkflowSecurityContext] = None,
+        image_pull_secrets: Optional[List[str]] = None,
+        workflow_template_ref: Optional[str] = None,
     ):
         self.name = f'{name.replace("_", "-")}-{str(uuid4()).split("-")[0]}'  # RFC1123
         self.namespace = namespace or 'default'
@@ -59,6 +76,8 @@ class Workflow:
         self.security_context = security_context
         self.service_account_name = service_account_name
         self.labels = labels
+        self.image_pull_secrets = image_pull_secrets
+        self.workflow_template_ref = workflow_template_ref
 
         self.dag_template = IoArgoprojWorkflowV1alpha1DAGTemplate(tasks=[])
         self.template = IoArgoprojWorkflowV1alpha1Template(
@@ -68,9 +87,19 @@ class Workflow:
             parallelism=self.parallelism,
         )
 
-        self.spec = IoArgoprojWorkflowV1alpha1WorkflowSpec(
-            templates=[self.template], entrypoint=self.name, volumes=[], volume_claim_templates=[]
-        )
+        if self.workflow_template_ref:
+            self.workflow_template = IoArgoprojWorkflowV1alpha1WorkflowTemplateRef(name=self.workflow_template_ref)
+            self.spec = IoArgoprojWorkflowV1alpha1WorkflowSpec(
+                workflow_template_ref=self.workflow_template,
+                entrypoint=self.workflow_template_ref,
+                volumes=[],
+                volume_claim_templates=[],
+            )
+        else:
+            self.spec = IoArgoprojWorkflowV1alpha1WorkflowSpec(
+                templates=[self.template], entrypoint=self.name, volumes=[], volume_claim_templates=[]
+            )
+
         if self.security_context:
             security_context = self.security_context.get_security_context()
             setattr(self.spec, 'security_context', security_context)
@@ -79,6 +108,10 @@ class Workflow:
             setattr(self.template, 'service_account_name', self.service_account_name)
             setattr(self.spec, 'service_account_name', self.service_account_name)
 
+        if self.image_pull_secrets:
+            secret_refs = [LocalObjectReference(name=name) for name in self.image_pull_secrets]
+            setattr(self.spec, 'image_pull_secrets', secret_refs)
+
         self.metadata = ObjectMeta(name=self.name)
         if self.labels:
             setattr(self.metadata, 'labels', self.labels)
@@ -158,7 +191,19 @@ class Workflow:
         t.argo_task.dependencies = list(free_tasks)
 
     def submit(self, namespace: Optional[str] = None) -> IoArgoprojWorkflowV1alpha1Workflow:
-        """Submits the workflow"""
+        """Submits the workflow.
+
+        Notes
+        -----
+        This method is deprecated in favor of `workflow.create(...)`.
+        """
+        warnings.warn("`submit` is deprecated in favor of `create`", DeprecationWarning, stacklevel=2)
+        if namespace is None:
+            namespace = self.namespace
+        return self.service.submit(self.workflow, namespace)
+
+    def create(self, namespace: Optional[str] = None) -> IoArgoprojWorkflowV1alpha1Workflow:
+        """Creates the workflow"""
         if namespace is None:
             namespace = self.namespace
         return self.service.submit(self.workflow, namespace)
